#ifndef DECISIONDATABASE_H
#define DECISIONDATABASE_H


#include "ros/ros.h"
#include "../common/sqlite3helper.h"

namespace robot {
namespace decision {

class Database{

public :
    Database(){}

    static std::string Name();
    /**
    * @brief decision模块的数据库初始化，如果相关表不存在，则建立相关的表
    * @brief 相关的表:action_directive,location_map,location_rule_info
    * @return 0失败
    */
    static int InitDB();

    /**
    * @brief 向action_firective表中插入一条
    * @return 0失败
    */
    static int InsertActionDirective(const char* ruleName,
                              int iDirectiveID,
                              const char* directiveInfo,
                              int iDirectiveResult);

    /**
    * @brief 向location_map表中插入一条
    * @return 0失败
    */
    static int InsertLocationMap(float fLocationLon1,
                          float fLocationLat1,
                          float fLocationLon2,
                          float fLocationLat2,
                          float fLocationX1,
                          float fLocationY1,
                          const char * mapId ,
                          float fLocationX2,
                          float fLocationY2);

    /**
    * @brief 向location_rule_info表中插入一条
    * @return 0失败
    */
    static int InsertLocationRuleInfo(const char * ruleName,
                               int iLocationID,
                               double dLocationLon,
                               double dLocationLat,
                               double dLocationHeight,
                               float fLocationSpeed
                               );

    /**
    * @brief 在action_firective表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    static int DeleteActionDirective(const char* ruleName);

    /**
    * @brief 在action_firective表中,
    * @brief 删除directive_id的记录
    * @return 0失败
    */
    static int DeleteActionDirective(int iDirectiveID);

    /**
    * @brief 在location_map表中,
    * @brief 删除map_id的记录
    * @return 0失败
    */
    static int DeleteLocationMap(const char* mapID);

    /**
    * @brief 在location_rule_info表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    static int DeleteLocationRuleInfo(const char* RuleName);

    /**
    * @brief 在location_rule_info表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    static int DeleteLocationRuleInfo(int iLocationID);

    /**
    * @brief 修改rule_name和directive_id的directive_result的记录
    * @return 0失败
    */
    static int UpdateActionDirective(const char* ruleName,int iDirectiveID,int iNewDirectiveResult);

    static int UpdateLocationMap(float fNewLocationLon1,
                          float fNewLocationLat1,
                          float fNewLocationLon2,
                          float fNewLocationLat2,
                          float fNewLocationX1,
                          float fNewLocationY1,
                          float fNewLocationX2,
                          float fNewLocationY2,
                          const char * mapId);
    };

}
}










#endif // DECISIONDATABASE_H
